@@ -24,9 +24,21 @@ void lshift(int n,unsigned short *pm,int m)
 { 
   unsigned short *pa,*pc;
   
-  pc=pm+m-1; pa=pm+n/16; m=n%16; n=16-m;
-  for(; pa<pc ;){ *pm=(*pa++)<<m; *pm++ |= *pa>>n;}
-  *pm++ = *pa<<m; while(pm<=pc) *pm++ =0;
+  pc=pm+m-1; 
+  if (n<16*m)
+    {
+      pa=pm+n/16; 
+      m=n%16; 
+      n=16-m;
+      while (pa<pc)
+	{ 
+	  *pm=(*pa++)<<m; 
+	  *pm++ |= *pa>>n;
+	}
+      *pm++ = *pa<<m; 
+    }
+  while(pm<=pc) 
+    *pm++ =0;
 }
 
 void rshift(int n,unsigned short *pm,int m)
@@ -34,9 +46,18 @@ void rshift(int n,unsigned short *pm,int m)
   unsigned short *pa,*pc;
   
   pc=pm+m-1;
-  if(n<16*m){ pa=pc-n/16; m=n%16; n=16-m;
-    for(; pa>pm ;){ *pc=(*pa--)>>m; *pc-- |= *pa<<n;}
-    *pc-- = *pa>>m;
-   }
-  while(pc>=pm) *pc-- =0;
+  if(n<16*m)
+    { 
+      pa=pc-n/16; 
+      m=n%16; 
+      n=16-m;
+      while (pa>pm)
+	{ 
+	  *pc=(*pa--)>>m; 
+	  *pc-- |= *pa<<n;
+	}
+      *pc-- = *pa>>m;
+    }
+  while(pc>=pm) 
+    *pc-- =0;
 }
