@@ -28,8 +28,13 @@ struct xpr xdiv(struct xpr s,struct xpr t)
   unsigned short *pc,e,i;
   
   pc=(unsigned short *)&t; e= *pc; *pc=bias;
-  a=dbltox(1/xtodbl(t)); *pc=e;
-  pc=(unsigned short *)&a; *pc+=bias-(e&m_exp); *pc|=e&m_sgn;
-  for(i=0; i<itt_div ;++i) a=xmul(a,xadd(two,xmul(a,t),1));
-  return xmul(s,a);
+  if ( (xsigerr (xprcmp(&t, &zero) == 0, XEDIV, "xdiv()")) )
+    return zero;
+  else
+    {
+      a=dbltox(1/xtodbl(t)); *pc=e;
+      pc=(unsigned short *)&a; *pc+=bias-(e&m_exp); *pc|=e&m_sgn;
+      for(i=0; i<itt_div ;++i) a=xmul(a,xadd(two,xmul(a,t),1));
+      return xmul(s,a);
+    }
 }
