@@ -22,19 +22,58 @@
 
 #include "xpre.h"
 
+struct xpr xexp2(struct xpr x)
+{
+  struct xpr s, d, f;
+  unsigned short* pf = (unsigned short*) &x;
+  int m, k;
+
+  if ( xprcmp (&x, &xE2min) < 0 )
+    return zero;
+  else if ( (xsigerr (xprcmp (&x, &xE2max) > 0, XFPOFLOW, NULL)) )
+    return x_huge;
+  else
+    {
+      m = (*pf & m_sgn) ? 1 : 0;
+      x = sfmod (x, &k);  
+      if ((m))
+	k *= -1;
+      /* -bias <= k <= +bias */
+      x = xmul (x, ln2);
+      if(xex(&x)> -bias)
+	{
+	  x=xpr2(x,-1); 
+	  s=xmul(x,x); 
+	  f=zero;
+	  for(d=inttox(m=ms_exp); m>1 ; m-=2, d=inttox(m))
+	    f=xdiv(s,xadd(d,f,0)); 
+	  f=xdiv(x,xadd(d,f,0));
+	  f=xdiv(xadd(d,f,0),xadd(d,f,1));
+	}
+      else 
+	f=one;      
+      pf = (unsigned short*) &f;
+      if (-k > *pf)
+	return zero;
+      else
+	{
+	  *pf += k;
+	  if ( (xsigerr(*pf >= m_exp, XFPOFLOW, NULL)) )
+	    return x_huge;
+	  else
+	    return f;
+	}
+    }
+}
+
 struct xpr xexp(struct xpr z)
 { 
-  struct xpr s,d,f; int m,k;
-  
-  if(neg(&z)) m=1; else m=0;
-  z=sfmod(z,&k); if(m) k= -k;
-  if(xex(&z)> -bias){
-    z=xpr2(z,-1); s=xmul(z,z); f=zero;
-    for(d=inttox(m=ms_exp); m>1 ;){
-      f=xdiv(s,xadd(d,f,0)); d=inttox(m-=2); }
-    f=xdiv(z,xadd(d,f,0));
-    f=xdiv(xadd(d,f,0),xadd(d,f,1));
-   }
-  else f=one;
-  if(k) return xmul(f,xpwr(ee,k)); else return f;
+  return xexp2 ( xmul (z, log2_e) );
 }
+
+struct xpr xexp10(struct xpr z)
+{ 
+  return xexp2 ( xmul (z, log2_10) );
+}
+
+
