@@ -27,14 +27,45 @@ struct xpr xlog(struct xpr z)
 { 
   struct xpr f,h; int k,m;
   
-  if(neg(&z) || xex(&z)== -bias){
-    fprintf(stderr, "xlog range error\n"); return zero;}
-  if(xprcmp(&z,&one)==0) return zero;
-  z=xfrex(z,&m); z=xmul(z,srt2);
-  z=xdiv(xadd(z,one,1),xadd(z,one,0));
-  h=xpr2(z,1); z=xmul(z,z);
-  for(f=h,k=1; xex(&h)> -max_p ;){
-    h=xmul(h,z); f=xadd(f,xdiv(h,inttox(k+=2)),0);
-   }
-  return xadd(f,xmul(ln2,dbltox(m-.5)),0);
+  if( (xsigerr ((neg(&z)) || xex(&z)== -bias, XEDOM, "xlog()")) ) 
+    return x_huge;
+  else if(xprcmp(&z,&one)==0) 
+    return zero;
+  else
+    {
+      z=xfrex(z,&m); z=xmul(z,srt2);
+      z=xdiv(xadd(z,one,1),xadd(z,one,0));
+      h=xpr2(z,1); z=xmul(z,z);
+      for(f=h,k=1; xex(&h)> -max_p ;){
+	h=xmul(h,z); 
+	f=xadd(f,xdiv(h,inttox(k+=2)),0);
+      }
+      return xadd(f,xmul(ln2,dbltox(m-.5)),0);
+    }
+}
+
+struct xpr xlog2(struct xpr z)
+{ 
+  struct xpr f,h; int k,m;
+  
+  if( (xsigerr ((neg(&z)) || xex(&z)== -bias, XEDOM, "xlog2()")) ) 
+    return x_huge;
+  else if(xprcmp(&z,&one)==0) 
+    return zero;
+  else
+    {
+      z=xfrex(z,&m); z=xmul(z,srt2);
+      z=xdiv(xadd(z,one,1),xadd(z,one,0));
+      h=xpr2(z,1); z=xmul(z,z);
+      for(f=h,k=1; xex(&h)> -max_p ;){
+	h=xmul(h,z); 
+	f=xadd(f,xdiv(h,inttox(k+=2)),0);
+      }
+      return xadd(xmul(f,log2_e),dbltox(m-.5),0);
+    }
+}
+
+struct xpr xlog10(struct xpr z)
+{
+  return xmul(xlog(z), log10_e);  
 }
