@@ -23,21 +23,50 @@
 #include "xpre.h"
 
 /*
-  Modified by Ivano Primi - 11/21/2004
-  Now 'pa' and 'pb' are pointers to constant structures.
+
+  Modified by Ivano Primi - 11/21/2004 and 03/05/2005
+
 */
 
 int xprcmp(const struct xpr *pa, const struct xpr *pb)
 { 
-  unsigned short e,k,*p,*q; int m;
+  unsigned short e, k, *p, *q, p0, q0; 
+  int m;
   
-  p=(unsigned short *)pa; e= *p&m_sgn;
-  q=(unsigned short *)pb; k= *q&m_sgn;
-  if(e && !k) return -1; if(!e && k) return 1;
-  if(e) m= -1; else m=1;
-  e= *p&m_exp; k= *q&m_exp;
-  if(e>k) return m; if(e<k) return -m;
-  for(e=0; *++p== *++q && e<XDIM ;++e);
-  if(e<XDIM){ if(*p> *q) return m; else return -m;}
-  return 0;
+  p=(unsigned short *)pa; 
+  q=(unsigned short *)pb; 
+  for (m=1; m <= XDIM && p[m] == 0; m++);
+  if ( m > XDIM && (*p & m_exp) < m_exp )
+    /* *pa is actually zero */
+    p0 = 0;
+  else
+    p0 = *p;
+  for (m=1; m <= XDIM && q[m] == 0; m++);
+  if ( m > XDIM && (*q & m_exp) < m_exp )
+    /* *pb is actually zero */
+    q0 = 0;
+  else
+    q0 = *q;
+  e= p0 & m_sgn; k= q0 & m_sgn;
+  if(e && !k) 
+    return -1; 
+  else if(!e && k) 
+    return 1;
+  else /* *pa and *pb have the same sign */
+    {
+      m = (e) ? -1 : 1; 
+      e= p0 & m_exp; k= q0 & m_exp;
+      if(e>k) 
+	return m; 
+      else if(e<k) 
+	return -m;
+      else
+	{
+	  for(e=0; *++p== *++q && e<XDIM ;++e);
+	  if(e<XDIM)
+	    return (*p> *q ? m : -m);
+	  else
+	    return 0;
+	}
+    }
 }
