@@ -27,16 +27,59 @@ struct xpr xpwr(struct xpr s,int n)
   struct xpr t; unsigned k,m;
   
   t=one;
-  if(n<0){ m= -n; s=xdiv(one,s);} else m=n;
-  if(m){ for(k=1; k<=m ;k<<=1){
-           if((k&m)) t=xmul(s,t); s=xmul(s,s); }
-   }
+  if(n<0) 
+    { 
+      m= -n; 
+      if ( (xsigerr(xprcmp(&s, &zero) == 0, XEBADEXP, "xpwr()")) )
+	return zero;
+      s=xdiv(one,s);
+    } 
+  else 
+    m=n;
+  if( (m) )
+    { 
+      k = 1;
+      while (1)
+	{
+	  if((k&m)) t=xmul(s,t); 
+	  if((k<<=1) <= m)
+	    s=xmul(s,s);
+	  else
+	    break;
+	}
+    }
+  else
+    xsigerr(xprcmp(&s, &zero) == 0, XEBADEXP, "xpwr()");
   return t;
 }  
 
 struct xpr xpr2(struct xpr s,int m)
 { 
   unsigned short *p=(unsigned short *)&s;
-  
-  *p+=m; return s;
+  long e;
+
+  for (e=1; e <= XDIM && p[e] == 0; e++);
+  if ( e <= XDIM )
+    {
+      e = *p & m_exp; /* biased exponent */
+      if ( e+m < 0 )
+	return zero;
+      else if ( (xsigerr(e+m >= m_exp, XFPOFLOW, NULL)) )
+	return x_huge;
+      else
+	{
+	  *p+=m; 
+	  return s;
+	}
+    }
+  else /* s is zero or x_huge */
+    return s;
+}
+
+struct xpr xpow (struct xpr x, struct xpr y)
+{
+  if ( xsigerr ((neg(&x)) || xex(&x)== -bias, XEDOM, "xpow()") )
+    return zero;
+  else
+    return xexp2 (xmul (xlog2(x), y)); 
 }
