@@ -28,14 +28,18 @@ struct xpr xsqrt(struct xpr z)
 { 
   struct xpr s,h; short m,e; unsigned short *pc;
   
-  if(neg(&z)){ fprintf(stderr, "xsqrt range error\n"); return zero;}
-  pc=(unsigned short *)&z;
-  if(*pc==0) return zero;
-  e= *pc-bias; *pc=bias+(e%2); e/=2;
-  s=dbltox(sqrt(xtodbl(z)));
-  for(m=0; m<itt_div ;++m){
-    h=xdiv(xadd(z,xmul(s,s),1),xpr2(s,1));
-    s=xadd(s,h,0); }
-  pc=(unsigned short *)&s; *pc+=e;
-  return s;
+  if ( (xsigerr((neg(&z)), XEDOM, "xsqrt()"))  )
+    return zero;
+  else
+    {
+      pc=(unsigned short *)&z;
+      if(*pc==0) return zero;
+      e= *pc-bias; *pc=bias+(e%2); e/=2;
+      s=dbltox(sqrt(xtodbl(z)));
+      for(m=0; m<itt_div ;++m){
+	h=xdiv(xadd(z,xmul(s,s),1),xpr2(s,1));
+	s=xadd(s,h,0); }
+      pc=(unsigned short *)&s; *pc+=e;
+      return s;
+    }
 }
